'==============================================================
'===FIXED RATE LOGIC  TWEEN  DELTA SPIKE SUPPRESSION (DSS)===
'==============================================================
Type TFRLTimer
	Field newTime:Float = MilliSecs()
	Field oldTime:Float = MilliSecs()
	Field delta:Float

	Field dssOn:Int		' do we use delta spike suppression?
	Field dssIndex:Int		' index into DSS_Array where next delta value is written
	Field dssArray:Float[]	' this array contains the delta values to smooth
	Field dssLenArray:Int	' how big is the array of delta values

	Field logicFPS:Float
	Field accumulator:Float, tween:Float
	
	Field fpsAccumulator:Float
	Field updateCount:Int
	Field renderCount:Int
	Field updatesPerSecond:Int
	Field rendersPerSecond:Int
	

	Function CreateFRL:TFRLTimer(logicCyclesPerSec:Float, numSamples:Int = 0)
		Local frl:TFRLTimer = New TFRLTimer
		frl.logicFPS = 1.0 / logicCyclesPerSec
		If numSamples
			frl.dssOn = True
			frl.dssArray = New Float[numSamples]
			frl.dssLenArray = numSamples
		EndIf
		Return frl
	End Function

	Method ProcessTime:Float()
		Self.newTime = MilliSecs()
		Self.delta = Float (Self.newTime - Self.oldTime) * 0.001
		Self.oldTime = Self.newTime
		
		If Self.dssOn = True
			Self.dssArray[Self.dssIndex] = Self.delta
			
			Local smoothDelta:Float = 0
			
			For Local i:Int = 0 To Self.dssLenArray - 1
				smoothDelta:+Self.dssArray[i]
			Next
			Self.delta = Float smoothDelta / dssLenArray
			
			Self.dssIndex:+1
			If Self.dssIndex > dssLenArray - 1 Then Self.dssIndex = 0
		EndIf
		
		Self.accumulator:+Self.delta
		
		Self.fpsAccumulator:+Self.delta
		If Self.fpsAccumulator > 1.0
			Self.fpsAccumulator:-1.0
			Self.updatesPerSecond = Self.updateCount
			Self.updateCount = 0
			Self.rendersPerSecond = Self.renderCount
			Self.renderCount = 0
		End If
		
		Return Self.delta
	End Method

	
	Method LogicUpdateRequired:Int()
		If Self.accumulator > Self.logicFPS
			Self.updateCount:+1
			Self.accumulator:-Self.logicFPS
			Return True
		End If
		Return False
	End Method

	
	Method GetLogicFPS:Float()
		Return Self.logicFPS
	End Method
	
	
	Method GetTween:Float()
		Self.renderCount:+1
		Return Self.accumulator / Self.logicFPS
	End Method
	

	Method ShowSpikeSuppression(x:Int, y:Int)
		DrawText "Delta Spike Suppressor:", x, y
		For Local i:Int = 0 To Self.dssLenArray - 1
			DrawText Self.dssArray[i], x, y + (i + 1) * 20
		Next
		DrawText "Final Delta: " + Self.delta, x, y + (Self.dssLenArray + 1) * 20
	End Method
	
	
	Method ShowFPS(x:Int, y:Int, showUpdateFPS:Int = True, showRenderFPS:Int = True)
		Local ty = y
		
		If showUpdateFPS
			DrawText "Logic FPS:  " + updatesPerSecond, x, ty
			ty:+20
		End If
		If showRenderFPS
			DrawText "Render FPS: " + rendersPerSecond, x, ty
		End If
	End Method
End Type

